// [[Rcpp::depends(RcppEigen)]]
// [[Rcpp::interfaces(r,cpp)]]
#include <RcppEigen.h>
#include <Rcpp.h>

using namespace Rcpp;

//typedef Eigen::SparseMatrix<double> SpMat;
//typedef Eigen::SparseMatrix<double>::InnerIterator InIterMat;

/*****  Center and standardize  *****/
// [[Rcpp::export]]
List scaleC(Eigen::MatrixXd X){
  int i, p=X.cols(), N=X.rows();
  Eigen::VectorXd mX(p), sdX(p);
  for (i=0;i<p;++i) {
    mX(i)=X.col(i).mean();
    X.col(i)=X.col(i).array()-mX(i);
    sdX(i)=sqrt(X.col(i).squaredNorm()/N);
    X.col(i)/=sdX(i);
  }
  return List::create(Named("x")=X, Named("sd")=sdX, Named("m")=mX);
}

/*****  Omega  *****/
// [[Rcpp::export]]
List OmegaC(Eigen::MatrixXd & Omega, Eigen::VectorXi & sgn){
  int i, j, p=sgn.size();
  Eigen::VectorXi nadj=Eigen::VectorXi::Zero(p);
  Eigen::VectorXd ndegree=Eigen::VectorXd::Zero(p);
  
  //Omega.diagonal().setZero();
  Eigen::SparseMatrix<double> OmegaS=Omega.sparseView();
  
  for(i=0;i<p;++i){
    for(Eigen::SparseMatrix<double>::InnerIterator it(OmegaS, i);it;++it){
      ++nadj(i);
      ndegree(i)+=it.value();
    }
  }
  
  Eigen::MatrixXi loc=Eigen::MatrixXi::Zero(nadj.maxCoeff(), p);
  for(i=0;i<p;++i){
    j=0;
    for(Eigen::SparseMatrix<double>::InnerIterator it(OmegaS, i);it;++it){
      loc(j++, i)=it.index();
      OmegaS.coeffRef(it.index(), i)=it.value()*sgn(i)*sgn(it.index())/sqrt(ndegree(i)*ndegree(it.index()));
    }
  }
  
  return(List::create(Named("nadj")=nadj, Named("loc")=loc, Named("Omega")=OmegaS));
}

/*****  Sparse Omega  *****/
// [[Rcpp::export]]
List OmegaSC(Eigen::SparseMatrix<double> & OmegaS, Eigen::VectorXi & sgn){
  int i, j, p=sgn.size();
  Eigen::VectorXi nadj=Eigen::VectorXi::Zero(p);
  Eigen::VectorXd ndegree=Eigen::VectorXd::Zero(p);
  
  for(i=0;i<p;++i){
    for(Eigen::SparseMatrix<double>::InnerIterator it(OmegaS, i);it;++it){
      ++nadj(i);
      ndegree(i)+=it.value();
    }
  }
  
  Eigen::MatrixXi loc=Eigen::MatrixXi::Zero(nadj.maxCoeff(), p);
  for(i=0;i<p;++i){
    j=0;
    for(Eigen::SparseMatrix<double>::InnerIterator it(OmegaS, i);it;++it){
      loc(j++, i)=it.index();
      OmegaS.coeffRef(it.index(), i)=it.value()*sgn(i)*sgn(it.index())/sqrt(ndegree(i)*ndegree(it.index()));
    }
  }
  
  return(List::create(Named("nadj")=nadj, Named("loc")=loc, Named("Omega")=OmegaS));
}





/////////////////////////////////
/////   Linear regression   /////
/////////////////////////////////

/*****  LM: Lambda path (max) inner product <xj,y> *****/
// [[Rcpp::export]]
double maxLambdaLmC(Eigen::MatrixXd X, Eigen::VectorXd y, double alpha, Eigen::VectorXd wbeta, int N0){
  Eigen::VectorXd Li(N0);
  
  Li=(y.transpose()*X).cwiseAbs()/N0;   // <xj,y>/N0
  Li=Li.array()/wbeta.array()/alpha ;
  return Li.maxCoeff();
}

/*****  LM: Lambda path (max) inner product <xj,y> *****/
// [[Rcpp::export]]
double maxLambdaLmiC(Eigen::MatrixXd X, Eigen::VectorXd y, double alpha, Eigen::VectorXd wbeta, int N0){
  Eigen::VectorXd Li(N0);
  int p=X.cols();
  
  Li=(y.transpose()*X).cwiseAbs()/N0;   // <xj,y>/N0
  Li=Li.array()/wbeta.array()/alpha ;
  return Li.tail(p-1).maxCoeff();
}


/*****  Used for CV trimming  *****/   //  
// [[Rcpp::export]]
Eigen::VectorXd cvTrimLmC(Eigen::VectorXd beta, int nn, int nn2, Eigen::VectorXi loco, 
Eigen::MatrixXd XF, Eigen::VectorXd yF, int NF) {
  int i, j;
  Eigen::VectorXd RSS, xbF=Eigen::VectorXd::Zero(NF); // xb=Eigen::VectorXd::Zero(N),
  
  if(nn2>0){
    RSS.setZero(nn2); //nn2= # of part of data
    
    if(nn==0){
      RSS(0)=yF.squaredNorm();
    }else{
      for(i=0;i<nn;i++){
        j=loco(i); //   index of nonzero beta
        xbF+=XF.col(j)*beta(i); // 
        RSS(i)=(yF-xbF).squaredNorm();
      }
    }
    if(nn2>nn){for(i=nn;i<nn2;i++){RSS(i)=RSS(nn-1);}}
  }else{
    RSS.setZero(1);
    RSS(0)=yF.squaredNorm();
  }
  
  return(RSS);
}


/*****  LM: Enet (L1+L2)  *****/
// [[Rcpp::export]]
List EnetLmC(Eigen::MatrixXd X, Eigen::VectorXd y,
             double alpha, Eigen::VectorXd lambda, int nlambda, int ilambda, Eigen::VectorXd wbeta,
             int isd, int p, int N0, double thresh, int maxit){ 
  
  int  i, j, it=0, il, iadd, ia=0; 
  double lambda2, zi, obj0, obj1, b0, db0, objQi, objQj, rss0, rss1, RSS0;
  double lambdaMax;
  Eigen::VectorXd beta0=Eigen::VectorXd::Zero(p);
  Eigen::MatrixXd Beta=Eigen::MatrixXd::Zero(p,nlambda);
  Eigen::VectorXd lambda1(p);
  Eigen::VectorXi active=Eigen::VectorXi::Zero(p), iactive=Eigen::VectorXi::Zero(p);
  Eigen::VectorXi flag=Eigen::VectorXi::Zero(nlambda);
  Eigen::VectorXd RSS(nlambda), RSQ(nlambda);
  double xr, dbMax, thresh2=1e-5;
  Eigen::VectorXd mX(p), sdX(p), di(p);
  
  for (i=0;i<p;++i) {
    mX(i)=X.col(i).mean();
    X.col(i)=X.col(i).array()-mX(i);
    sdX(i)=sqrt(X.col(i).squaredNorm()/N0);
    X.col(i)/=sdX(i);
  }
  y=y.array()-y.mean();
  
  if (ilambda == 1) {
    if (alpha > 0.0) {
      lambdaMax=maxLambdaLmC(X, y, alpha, wbeta, N0)+0.0001;
    } else {
      lambdaMax=maxLambdaLmC(X, y, 0.001, wbeta, N0)+0.0001;
    }
    lambda=lambda.array()*lambdaMax;
  }
  
  RSS0=y.squaredNorm();
  obj0=RSS0/N0/2.0; rss0=RSS0;
  
  for(i=0;i<p;++i){
    di(i)=std::abs(y.dot(X.col(i))/N0);
  }
  
  for(il=0;il<nlambda;++il){
    lambda1=lambda(il)*alpha*wbeta;lambda2=lambda(il)*(1-alpha); // lambda1:vector lambda*alpha, lambda2=lambda*(1-alpha)
    
    for(i=0;i<p;++i){
      if(iactive(i)==0){
        if(di(i)>lambda1(i)){  
          active(ia)=i;iactive(i)=1;++ia;
        }
      }
    }
    
    local:;
    while(1){
      ++it;
      
      objQi=0.0; objQj=0.0; dbMax=0.0; rss1=rss0; obj1=obj0;
      for(i=0;i<ia;++i){
        j=active(i);
        xr=y.dot(X.col(j));
        zi=xr/N0+beta0(j);
        if(zi>lambda1(j)){
          b0=(zi-lambda1(j))/(lambda2+1); // x*x/N=1
          db0=beta0(j)-b0;beta0(j)=b0;
          y+=db0*X.col(j); 
          objQj+=std::abs(b0)*lambda1(j);
          objQi+=pow(b0, 2);
        }else if(zi<-lambda1(j)){
          b0=(zi+lambda1(j))/(lambda2+1);
          db0=beta0(j)-b0;beta0(j)=b0;
          y+=db0*X.col(j);
          objQj+=std::abs(b0)*lambda1(j);
          objQi+=pow(b0, 2);
        }else{
          b0=0.0; db0=0.0;
          if(beta0(j)!=b0){
            db0=beta0(j)-b0;beta0(j)=b0;
            y+=db0*X.col(j);
          }
        }
        
        rss0+=db0*(db0*N0+2.0*xr);
        dbMax=std::max(dbMax, pow(db0, 2));
      }//for update
      
      obj0=rss0/N0/2.0+objQj+objQi*lambda2/2.0;
      
      if(std::abs(rss0-rss1)<std::abs(thresh2*rss1)){flag(il)=0;break;}
      if(rss0 < RSS0*0.001){flag(il)=0;break;}
      if(dbMax<thresh){flag(il)=0;break;}
      if(std::abs(obj1-obj0)<std::abs(thresh2*obj1)){flag(il)=0;break;}
      if(obj0!=obj0){flag(il)=2; ++il; goto exit;}
      if(it>=maxit){flag(il)=1; ++il; goto exit;}
    }//while
    
    iadd=0;
    for(i=0;i<p;++i){
      if(iactive(i)==0){
        di(i)=std::abs(y.dot(X.col(i))/N0);
        if(di(i)>lambda1(i)){
          active(ia)=i;iactive(i)=1;++ia;iadd=1;
        }
      }
    }
    if(iadd==1){goto local;}
    
    if (isd == 1) {
      Beta.col(il)=beta0; 
    } else {
      Beta.col(il)=beta0.array()/sdX.array();
    }
    RSS(il)=rss0; RSQ(il)=1.0-rss0/RSS0;
    
    if(RSQ(il) > 0.999) {++il; goto exit;}
  }//for lambda
  
  exit:;
  return(List::create(Named("Beta")=Beta.block(0,0,p,il), Named("flag")=flag.head(il), Named("rsq")=RSQ.head(il),
                      Named("RSS")=RSS.head(il), Named("lambda")=lambda.head(il), Named("nlambda")=il));
}


/*****  LM: Enet (L1+L2) cross-validation  *****/
// [[Rcpp::export]]
List cvEnetLmC(Eigen::MatrixXd X, Eigen::VectorXd y,
double alpha, Eigen::VectorXd lambda, int nlambda, Eigen::VectorXd wbeta,
int N, int p, double thresh, int maxit, Eigen::MatrixXd XF, Eigen::VectorXd yF, int NF){
  
  int i, j, it=0, il, iadd, ia=0; 
  double lambda2, zi, obj0, obj1, rss0, rss1, b0, db0, objQi, objQj, RSS0;
  Eigen::VectorXd beta0=Eigen::VectorXd::Zero(p);
  Eigen::MatrixXd Beta=Eigen::MatrixXd::Zero(p,nlambda); // beta matrix for different lambdas
  Eigen::VectorXd lambda1(p);
  Eigen::VectorXd RSSp(nlambda), RSS(nlambda), RSQ(nlambda);
  Eigen::VectorXi active=Eigen::VectorXi::Zero(p), iactive=Eigen::VectorXi::Zero(p);
  Eigen::VectorXi flag=Eigen::VectorXi::Zero(nlambda);
  Eigen::VectorXd xb=Eigen::VectorXd::Zero(N);
  Eigen::VectorXd xbF(NF);
  double xr, dbMax, thresh2=1e-5;
  Eigen::VectorXd mX(p), sdX(p), di(p);
  
  for (i=0;i<p;++i) {
    mX(i)=X.col(i).mean();
    X.col(i)=X.col(i).array()-mX(i);
    sdX(i)=sqrt(X.col(i).squaredNorm()/N);
    X.col(i)/=sdX(i);
  }
  y=y.array()-y.mean();
  
  RSS0=y.squaredNorm();
  obj0=RSS0/N/2.0; rss0=RSS0;
  
  for(i=0;i<p;++i){
    di(i)=std::abs(y.dot(X.col(i))/N);
  }
  
  for(il=0;il<nlambda;++il){
    lambda1=lambda(il)*alpha*wbeta;lambda2=lambda(il)*(1-alpha); // lambda1:vector lambda*alpha, lambda2=lambda*(1-alpha) 
    
      for(i=0;i<p;++i){
        if(iactive(i)==0){
          if(di(i)>lambda1(i)){
            active(ia)=i;iactive(i)=1;++ia;iadd=1;
          }
        }
      }
    
    //it=0;
    local:
      while(1){
        ++it;
        
        objQi=0.0; objQj=0.0; dbMax=0.0; rss1=rss0; obj1=obj0;
        for(i=0;i<ia;++i){
          j=active(i);
          xr=y.dot(X.col(j));
          zi=xr/N+beta0(j);
          if(zi>lambda1(j)){
            b0=(zi-lambda1(j))/(lambda2+1); // x*x/N=1
            db0=beta0(j)-b0;beta0(j)=b0;
            y+=db0*X.col(j); 
            objQj+=std::abs(b0)*lambda1(j);
            objQi+=pow(b0, 2);
          }else if(zi<-lambda1(j)){
            b0=(zi+lambda1(j))/(lambda2+1);
            db0=beta0(j)-b0;beta0(j)=b0;
            y+=db0*X.col(j);
            objQj+=std::abs(b0)*lambda1(j);
            objQi+=pow(b0, 2);
          }else{
            b0=0.0; db0=0.0;
            if(beta0(j)!=b0){
              db0=beta0(j)-b0;beta0(j)=b0;
              y+=db0*X.col(j);
            }
          }
          
          rss0+=db0*(db0*N+2.0*xr);
          dbMax=std::max(dbMax, pow(db0, 2));
        }//for update
        
        obj0=rss0/N/2.0+objQj+objQi*lambda2/2.0;
        
        if(std::abs(rss0-rss1)<std::abs(thresh2*rss1)){flag(il)=0;break;}
        if(rss0 < RSS0*0.001){flag(il)=0;break;}
        if(dbMax<thresh){flag(il)=0;break;}
        if(std::abs(obj1-obj0)<std::abs(thresh2*obj1)){flag(il)=0;break;}
        if(obj0!=obj0){flag(il)=2;goto exit;}
        if(it>=maxit){flag(il)=1;goto exit;}
      }//while
      
      iadd=0;
      for(i=0;i<p;++i){
        if(iactive(i)==0){
          di(i)=std::abs(y.dot(X.col(i))/N);
          if(di(i)>lambda1(i)){
            active(ia)=i;iactive(i)=1;++ia;iadd=1;
          }
        }
      }
    if(iadd==1){goto local;}
    
    Beta.col(il)=beta0.array()/sdX.array(); RSS(il)=rss0; RSQ(il)=1.0-rss0/RSS0;
    
    xbF.setZero(NF);
    for(i=0;i<ia;i++){j=active(i);xbF+=XF.col(j)*(beta0(j)/sdX(j));}
    RSSp(il)=(yF-xbF).squaredNorm();
    
    //if(RSQ(il) > 0.999) goto exit;
  }//for lambda
  
  exit:
  return(List::create(Named("Beta")=Beta, Named("flag")=flag,
  Named("RSS")=RSS, Named("rsq")=RSQ, Named("RSSp")=RSSp, Named("nlambda")=il));
}



////////////////////////////////////////////////
/////   Linear regression - non-negative   /////
////////////////////////////////////////////////

/*****  LM: Enet (L1+L2)  *****/
// [[Rcpp::export]]
List EnetLmNNC(Eigen::MatrixXd X, Eigen::VectorXd y,
             double alpha, Eigen::VectorXd lambda, int nlambda, int ilambda, Eigen::VectorXd wbeta,
             int isd, int p, int N0, double thresh, int maxit){ 
  
  int  i, j, it=0, il, iadd, ia=0; 
  double lambda2, zi, obj0, obj1, b0, db0, objQi, objQj, rss0, rss1, RSS0;
  double lambdaMax;
  Eigen::VectorXd beta0=Eigen::VectorXd::Zero(p);
  Eigen::MatrixXd Beta=Eigen::MatrixXd::Zero(p,nlambda);
  Eigen::VectorXd lambda1(p);
  Eigen::VectorXi active=Eigen::VectorXi::Zero(p), iactive=Eigen::VectorXi::Zero(p);
  Eigen::VectorXi flag=Eigen::VectorXi::Zero(nlambda);
  Eigen::VectorXd RSS(nlambda), RSQ(nlambda);
  double xr, dbMax, thresh2=1e-5;
  Eigen::VectorXd mX(p), sdX(p), di(p);
  
  for (i=0;i<p;++i) {
    mX(i)=X.col(i).mean();
    X.col(i)=X.col(i).array()-mX(i);
    sdX(i)=sqrt(X.col(i).squaredNorm()/N0);
    X.col(i)/=sdX(i);
  }
  y=y.array()-y.mean();
  
  if (ilambda == 1) {
    if (alpha > 0.0) {
      lambdaMax=maxLambdaLmC(X, y, alpha, wbeta, N0)+0.0001;
    } else {
      lambdaMax=maxLambdaLmC(X, y, 0.001, wbeta, N0)+0.0001;
    }
    lambda=lambda.array()*lambdaMax;
  }
  
  RSS0=y.squaredNorm();
  obj0=RSS0/N0/2.0; rss0=RSS0;
  
  for(i=0;i<p;++i){
    // di(i)=std::abs(y.dot(X.col(i))/N0);
    di(i)=(y.dot(X.col(i))/N0);
  }
  
  for(il=0;il<nlambda;++il){
    lambda1=lambda(il)*alpha*wbeta;lambda2=lambda(il)*(1-alpha); // lambda1:vector lambda*alpha, lambda2=lambda*(1-alpha)
    
    for(i=0;i<p;++i){
      if(iactive(i)==0){
        if(di(i)>lambda1(i)){  
          active(ia)=i;iactive(i)=1;++ia;
        }
      }
    }
    
    local:
      while(1){
        ++it;
        
        objQi=0.0; objQj=0.0; dbMax=0.0; rss1=rss0; obj1=obj0;
        for(i=0;i<ia;++i){
          j=active(i);
          xr=y.dot(X.col(j));
          zi=xr/N0+beta0(j);
          if(zi>lambda1(j)){
            b0=(zi-lambda1(j))/(lambda2+1); // x*x/N=1
            if (b0 < 0.0) b0=0.0; // non-negative
            db0=beta0(j)-b0;beta0(j)=b0;
            y+=db0*X.col(j); 
            objQj+=std::abs(b0)*lambda1(j);
            objQi+=pow(b0, 2);
          }else if(zi<-lambda1(j)){
            b0=(zi+lambda1(j))/(lambda2+1);
            if (b0 < 0.0) b0=0.0; // non-negative
            db0=beta0(j)-b0;beta0(j)=b0;
            y+=db0*X.col(j);
            objQj+=std::abs(b0)*lambda1(j);
            objQi+=pow(b0, 2);
          }else{
            b0=0.0; db0=0.0;
            if(beta0(j)!=b0){
              db0=beta0(j)-b0;beta0(j)=b0;
              y+=db0*X.col(j);
            }
          }
          
          rss0+=db0*(db0*N0+2.0*xr);
          dbMax=std::max(dbMax, pow(db0, 2));
        }//for update
        
        obj0=rss0/N0/2.0+objQj+objQi*lambda2/2.0;
        
        if(std::abs(rss0-rss1)<std::abs(thresh2*rss1)){flag(il)=0;break;}
        if(rss0 < RSS0*0.001){flag(il)=0;break;}
        if(dbMax<thresh){flag(il)=0;break;}
        if(std::abs(obj1-obj0)<std::abs(thresh2*obj1)){flag(il)=0;break;}
        if(obj0!=obj0){flag(il)=2;goto exit;}
        if(it>=maxit){flag(il)=1;goto exit;}
      }//while
      
      iadd=0;
    for(i=0;i<p;++i){
      if(iactive(i)==0){
        // di(i)=std::abs(y.dot(X.col(i))/N0);
        di(i)=(y.dot(X.col(i))/N0);
        if(di(i)>lambda1(i)){
          active(ia)=i;iactive(i)=1;++ia;iadd=1;
        }
      }
    }
    if(iadd==1){goto local;}
    
    if (isd == 1) {
      Beta.col(il)=beta0; 
    } else {
      Beta.col(il)=beta0.array()/sdX.array();
    }
    RSS(il)=rss0; RSQ(il)=1.0-rss0/RSS0;
    
    if(RSQ(il) > 0.999) goto exit;
  }//for lambda
  
  exit:
    return(List::create(Named("Beta")=Beta, Named("flag")=flag, Named("rsq")=RSQ,
                              Named("RSS")=RSS, Named("lambda")=lambda, Named("nlambda")=il));
}



/*****  LM: Enet (L1+L2) cross-validation  *****/
// [[Rcpp::export]]
List cvEnetLmNNC(Eigen::MatrixXd X, Eigen::VectorXd y,
               double alpha, Eigen::VectorXd lambda, int nlambda, Eigen::VectorXd wbeta,
               int N, int p, double thresh, int maxit, Eigen::MatrixXd XF, Eigen::VectorXd yF, int NF){
  
  int i, j, it=0, il, iadd, ia=0; 
  double lambda2, zi, obj0, obj1, rss0, rss1, b0, db0, objQi, objQj, RSS0;
  Eigen::VectorXd beta0=Eigen::VectorXd::Zero(p);
  Eigen::MatrixXd Beta=Eigen::MatrixXd::Zero(p,nlambda); // beta matrix for different lambdas
  Eigen::VectorXd lambda1(p);
  Eigen::VectorXd RSSp(nlambda), RSS(nlambda), RSQ(nlambda);
  Eigen::VectorXi active=Eigen::VectorXi::Zero(p), iactive=Eigen::VectorXi::Zero(p);
  Eigen::VectorXi flag=Eigen::VectorXi::Zero(nlambda);
  Eigen::VectorXd xb=Eigen::VectorXd::Zero(N);
  Eigen::VectorXd xbF(NF);
  double xr, dbMax, thresh2=1e-5;
  Eigen::VectorXd mX(p), sdX(p), di(p);
  
  for (i=0;i<p;++i) {
    mX(i)=X.col(i).mean();
    X.col(i)=X.col(i).array()-mX(i);
    sdX(i)=sqrt(X.col(i).squaredNorm()/N);
    X.col(i)/=sdX(i);
  }
  y=y.array()-y.mean();
  
  RSS0=y.squaredNorm();
  obj0=RSS0/N/2.0; rss0=RSS0;
  
  for(i=0;i<p;++i){
    di(i)=(y.dot(X.col(i))/N);
  }
  
  for(il=0;il<nlambda;++il){
    lambda1=lambda(il)*alpha*wbeta;lambda2=lambda(il)*(1-alpha); // lambda1:vector lambda*alpha, lambda2=lambda*(1-alpha) 
    
    for(i=0;i<p;++i){
      if(iactive(i)==0){
        if(di(i)>lambda1(i)){
          active(ia)=i;iactive(i)=1;++ia;iadd=1;
        }
      }
    }
    
    //it=0;
    local:
      while(1){
        ++it;
        
        objQi=0.0; objQj=0.0; dbMax=0.0; rss1=rss0; obj1=obj0;
        for(i=0;i<ia;++i){
          j=active(i);
          xr=y.dot(X.col(j));
          zi=xr/N+beta0(j);
          if(zi>lambda1(j)){
            b0=(zi-lambda1(j))/(lambda2+1); // x*x/N=1
            if (b0 < 0.0) b0=0.0; // non-negative
            db0=beta0(j)-b0;beta0(j)=b0;
            y+=db0*X.col(j); 
            objQj+=std::abs(b0)*lambda1(j);
            objQi+=pow(b0, 2);
          }else if(zi<-lambda1(j)){
            b0=(zi+lambda1(j))/(lambda2+1);
            if (b0 < 0.0) b0=0.0; // non-negative
            db0=beta0(j)-b0;beta0(j)=b0;
            y+=db0*X.col(j);
            objQj+=std::abs(b0)*lambda1(j);
            objQi+=pow(b0, 2);
          }else{
            b0=0.0; db0=0.0;
            if(beta0(j)!=b0){
              db0=beta0(j)-b0;beta0(j)=b0;
              y+=db0*X.col(j);
            }
          }
          
          rss0+=db0*(db0*N+2.0*xr);
          dbMax=std::max(dbMax, pow(db0, 2));
        }//for update
        
        obj0=rss0/N/2.0+objQj+objQi*lambda2/2.0;
        
        if(std::abs(rss0-rss1)<std::abs(thresh2*rss1)){flag(il)=0;break;}
        if(rss0 < RSS0*0.001){flag(il)=0;break;}
        if(dbMax<thresh){flag(il)=0;break;}
        if(std::abs(obj1-obj0)<std::abs(thresh2*obj1)){flag(il)=0;break;}
        if(obj0!=obj0){flag(il)=2;goto exit;}
        if(it>=maxit){flag(il)=1;goto exit;}
      }//while
      
      iadd=0;
    for(i=0;i<p;++i){
      if(iactive(i)==0){
        di(i)=(y.dot(X.col(i))/N);
        if(di(i)>lambda1(i)){
          active(ia)=i;iactive(i)=1;++ia;iadd=1;
        }
      }
    }
    if(iadd==1){goto local;}
    
    Beta.col(il)=beta0.array()/sdX.array(); RSS(il)=rss0; RSQ(il)=1.0-rss0/RSS0;
    
    xbF.setZero(NF);
    for(i=0;i<ia;i++){j=active(i);xbF+=XF.col(j)*(beta0(j)/sdX(j));}
    RSSp(il)=(yF-xbF).squaredNorm();
    
    //if(RSQ(il) > 0.999) goto exit;
  }//for lambda
  
  exit:
    return(List::create(Named("Beta")=Beta, Named("flag")=flag,
                        Named("RSS")=RSS, Named("rsq")=RSQ, Named("RSSp")=RSSp, Named("nlambda")=il));
}





////////////////////////////////////////////////////////////
/////   Linear regression - non-negative + intercept   /////
////////////////////////////////////////////////////////////

/*****  LM: Enet (L1+L2)  *****/
// [[Rcpp::export]]
List EnetLmNNiC(Eigen::MatrixXd X, Eigen::VectorXd y,
               double alpha, Eigen::VectorXd lambda, int nlambda, int ilambda, Eigen::VectorXd wbeta,
               int isd, int p, int N0, double thresh, int maxit){ 
  
  int  i, j, it=0, il, iadd, ia=0; 
  double lambda2, zi, obj0, obj1, b0, db0, objQi, objQj, rss0, rss1, RSS0;
  double lambdaMax;
  Eigen::VectorXd beta0=Eigen::VectorXd::Zero(p);
  Eigen::MatrixXd Beta=Eigen::MatrixXd::Zero(p,nlambda);
  Eigen::VectorXd lambda1(p);
  Eigen::VectorXi active=Eigen::VectorXi::Zero(p), iactive=Eigen::VectorXi::Zero(p);
  Eigen::VectorXi flag=Eigen::VectorXi::Zero(nlambda);
  Eigen::VectorXd RSS(nlambda), RSQ(nlambda);
  double xr, dbMax, thresh2=1e-5;
  Eigen::VectorXd mX(p), sdX(p), di(p);
  
  for (i=0;i<p;++i) {
    mX(i)=X.col(i).mean();
    // X.col(i)=X.col(i).array()-mX(i);
    sdX(i)=sqrt(X.col(i).squaredNorm()/N0);
    X.col(i)/=sdX(i);
  }
  // y=y.array()-y.mean();
  
  if (ilambda == 1) {
    if (alpha > 0.0) {
      lambdaMax=maxLambdaLmiC(X, y, alpha, wbeta, N0)+0.0001;
    } else {
      lambdaMax=maxLambdaLmiC(X, y, 0.001, wbeta, N0)+0.0001;
    }
    lambda=lambda.array()*lambdaMax;
  }
  
  RSS0=y.squaredNorm();
  obj0=RSS0/N0/2.0; rss0=RSS0;
  
  for(i=0;i<p;++i){
    // di(i)=std::abs(y.dot(X.col(i))/N0);
    di(i)=(y.dot(X.col(i))/N0);
  }
  
  for(il=0;il<nlambda;++il){
    lambda1=lambda(il)*alpha*wbeta;lambda2=lambda(il)*(1-alpha); // lambda1:vector lambda*alpha, lambda2=lambda*(1-alpha)
    
    for(i=0;i<p;++i){
      if(iactive(i)==0){
        if(di(i)>lambda1(i)){  
          active(ia)=i;iactive(i)=1;++ia;
        }
      }
    }
    
    local:
      while(1){
        ++it;
        
        objQi=0.0; objQj=0.0; dbMax=0.0; rss1=rss0; obj1=obj0;
        for(i=0;i<ia;++i){
          j=active(i);
          xr=y.dot(X.col(j));
          zi=xr/N0+beta0(j);
          if(zi>lambda1(j)){
            b0=(zi-lambda1(j))/(lambda2+1); // x*x/N=1
            if (b0 < 0.0) b0=0.0; // non-negative
            db0=beta0(j)-b0;beta0(j)=b0;
            y+=db0*X.col(j); 
            objQj+=std::abs(b0)*lambda1(j);
            objQi+=pow(b0, 2);
          }else if(zi<-lambda1(j)){
            b0=(zi+lambda1(j))/(lambda2+1);
            if (b0 < 0.0) b0=0.0; // non-negative
            db0=beta0(j)-b0;beta0(j)=b0;
            y+=db0*X.col(j);
            objQj+=std::abs(b0)*lambda1(j);
            objQi+=pow(b0, 2);
          }else{
            b0=0.0; db0=0.0;
            if(beta0(j)!=b0){
              db0=beta0(j)-b0;beta0(j)=b0;
              y+=db0*X.col(j);
            }
          }
          
          rss0+=db0*(db0*N0+2.0*xr);
          dbMax=std::max(dbMax, pow(db0, 2));
        }//for update
        
        obj0=rss0/N0/2.0+objQj+objQi*lambda2/2.0;
        
        if(std::abs(rss0-rss1)<std::abs(thresh2*rss1)){flag(il)=0;break;}
        if(rss0 < RSS0*0.001){flag(il)=0;break;}
        if(dbMax<thresh){flag(il)=0;break;}
        if(std::abs(obj1-obj0)<std::abs(thresh2*obj1)){flag(il)=0;break;}
        if(obj0!=obj0){flag(il)=2;goto exit;}
        if(it>=maxit){flag(il)=1;goto exit;}
      }//while
      
      iadd=0;
    for(i=0;i<p;++i){
      if(iactive(i)==0){
        // di(i)=std::abs(y.dot(X.col(i))/N0);
        di(i)=(y.dot(X.col(i))/N0);
        if(di(i)>lambda1(i)){
          active(ia)=i;iactive(i)=1;++ia;iadd=1;
        }
      }
    }
    if(iadd==1){goto local;}
    
    if (isd == 1) {
      Beta.col(il)=beta0; 
    } else {
      Beta.col(il)=beta0.array()/sdX.array();
    }
    RSS(il)=rss0; RSQ(il)=1.0-rss0/RSS0;
    
    if(RSQ(il) > 0.999) goto exit;
  }//for lambda
  
  exit:
    return(List::create(Named("Beta")=Beta, Named("flag")=flag, Named("rsq")=RSQ,
                              Named("RSS")=RSS, Named("lambda")=lambda, Named("nlambda")=il));
}




